---- Merging image.e, graphics.e with a lot or alterations

global constant BLACK = 0,  -- in graphics modes this is "transparent"
                BLUE  = 1,
                GREEN = 2,
                CYAN =  3,
                RED   = 4,
                MAGENTA = 5,
                BROWN = 6,
                WHITE = 7,
                GRAY  = 8,
                BRIGHT_BLUE = 9,
                BRIGHT_GREEN = 10,
                BRIGHT_CYAN = 11,
                BRIGHT_RED = 12,
                BRIGHT_MAGENTA = 13,
                YELLOW = 14,
                BRIGHT_WHITE = 15

global constant VC_COLOR = 1,
                VC_MODE  = 2,
                VC_LINES = 3,
                VC_COLUMNS = 4,
                VC_XPIXELS = 5,
                VC_YPIXELS = 6,
                VC_NCOLORS = 7,
                VC_PAGES = 8

global constant NO_CURSOR       = #2000,
         UNDERLINE_CURSOR       = #0607,
         THICK_UNDERLINE_CURSOR = #0507,
         HALF_BLOCK_CURSOR      = #0407,
         BLOCK_CURSOR           = #0007

constant M_SOUND          = 1,
         M_LINE           = 2,
         M_PALETTE        = 3,
         --M_PIXEL          = 4, -- obsolete
         M_GRAPHICS_MODE  = 5,
         M_CURSOR         = 6,
         M_WRAP           = 7,
         M_SCROLL         = 8,
         M_SET_T_COLOR    = 9,
         M_SET_B_COLOR    = 10,
         M_POLYGON        = 11,
         M_TEXTROWS       = 12,
         M_VIDEO_CONFIG   = 13,
         M_ELLIPSE        = 18,
         --M_GET_PIXEL      = 21,  -- obsolete
         M_GET_POSITION   = 25,
         M_ALL_PALETTE    = 27

constant M_PIXEL = 4,
         M_GET_PIXEL = 21

global function video_config()
-- return sequence of information on video configuration
-- {color?, mode, text lines, text columns, xpixels, ypixels, #colors, pages}
    return machine_func(M_VIDEO_CONFIG, 0)
end function

global function graphics_mode(integer m)
-- try to set up a new graphics mode
-- return 0 if successful, non-zero if failed
   return machine_func(M_GRAPHICS_MODE, m)
end function

global procedure bk_color(integer c)
-- set the background color to c - text or graphics modes
    machine_proc(M_SET_B_COLOR, c)
end procedure

global procedure text_color(integer c)
-- set the foreground text color to c - text or graphics modes
-- add 16 to get blinking
    machine_proc(M_SET_T_COLOR, c)
end procedure

global procedure all_palette(sequence s)
-- s is a sequence of the form: {{r,g,b},{r,g,b}, ...{r,g,b}}
-- that specifies new color intensities for the entire set of
-- colors in the current graphics mode.  
    machine_proc(M_ALL_PALETTE, s)
end procedure

global procedure polygon(integer c,
                         integer fill,
                         sequence xyarray)
-- draw a polygon using a certain color
-- fill the area if fill is TRUE
-- 3 or more vertices are given in xyarray
    machine_proc(M_POLYGON, {c, fill, xyarray})
end procedure

global procedure cursor(integer style)
-- choose a cursor style
    machine_proc(M_CURSOR, style)
end procedure

type graphics_point(sequence p)
    return length(p) = 2 and p[1] >= 0 and p[2] >= 0
end type

global function save_image(graphics_point top_left, graphics_point bottom_right)
    integer x, width
    sequence save
    
    x = top_left[1]
    width = bottom_right[1] - x + 1
    save = {}
    for y = top_left[2] to bottom_right[2] do
        save = append(save, machine_func(M_GET_PIXEL, {x, y, width}))
    end for
    return save
end function

global procedure display_image(graphics_point xy, sequence pixels)
    for i=1 to length(pixels) do
            machine_proc(M_PIXEL, {pixels[i], xy})
            xy[2] = xy[2] + 1
    end for
end procedure

